// ==============================================================
//	This file is part of Glest (www.glest.org)
//
//	Copyright (C) 2001-2005 Martio Figueroa
//
//	You can redistribute this code and/or modify it under 
//	the terms of the GNU General Public License as published 
//	by the Free Software Foundation; either version 2 of the 
//	License, or (at your option) any later version
// ==============================================================

#ifndef _GLEST_GAME_FACTION_H_
#define _GLEST_GAME_FACTION_H_

#include <vector>

#include "upgrade.h"
#include "texture.h"
#include "resource.h"

using std::vector;

using Shared::Graphics::Texture2D;

namespace Glest{ namespace Game{

class Unit;
class TechTree;
class FactionType;
class ProducibleType;
class RequirableType;
class CommandType;
class UnitType;

//control
enum ControlerType{
    ctClosed,
	ctCpu,
	ctCpuUltra,
	ctHuman
};

// ===============================
// 	class Faction
//
///	Each of the game players
// ===============================

class Faction{
private:
	int index;
	bool thisFaction;
    vector<Resource*> resources;
    vector<Resource*> store;
	vector<Faction*> allies;
	vector<Unit*> units;
    const FactionType *factionType;
    ControlerType control;
	UpgradeManager upgradeManager; 
	Texture2D *texture;

public:
    Faction(
		const FactionType *factionType, ControlerType control, TechTree *techTree, 
		int factionIndex, bool thisFaction);
	~Faction();

    //get
	const Resource *getResource(const ResourceType *rt) const;
	const Resource *getResource(int i) const			{return resources[i];}
	int getStoreAmount(const ResourceType *rt) const;
	const FactionType *getType() const			{return factionType;}
	int getIndex() const								{return index;}
	bool getCpuControl() const;
	bool getCpuUltraControl() const						{return control==ctCpuUltra;}
	Unit *getUnit(int i) const							{return units[i];}
	int getUnitCount() const							{return units.size();}		
	const UpgradeManager *getUpgradeManager() const		{return &upgradeManager;}
	const Texture2D *getTexture() const					{return texture;}

	//upgrades
	void startUpgrade(const UpgradeType *ut);
	void cancelUpgrade(const UpgradeType *ut);
	void finishUpgrade(const UpgradeType *ut);

	//cost application
	bool applyCosts(const ProducibleType *p);
	void applyDiscount(const ProducibleType *p, int discount);
	void applyStaticCosts(const ProducibleType *p);
	void applyStaticProduction(const ProducibleType *p);
	void deApplyCosts(const ProducibleType *p);
	void deApplyStaticCosts(const ProducibleType *p);
	void applyCostsOnInterval();
	bool checkCosts(const ProducibleType *pt);

	//reqs
	bool reqsOk(const RequirableType *rt);
	bool reqsOk(const CommandType *ct);

	//diplomacy
	void pushAlly(Faction *faction);
	bool isAlly(Faction *faction);

    //other
	void addUnit(Unit *unit);
	void removeUnit(Unit *unit);
	void addStore(const UnitType *unitType);
	void removeStore(const UnitType *unitType);

	//resources
	void incResourceAmount(const ResourceType *rt, int amount);
	void setResourceBalance(const ResourceType *rt, int balance);

private:
	void limitResourcesToStore();
	void resetResourceAmount(const ResourceType *rt);
};

}}//end namespace

#endif
