// ==============================================================
//	This file is part of Glest Shared Library (www.glest.org)
//
//	Copyright (C) 2001-2005 Martio Figueroa
//
//	You can redistribute this code and/or modify it under 
//	the terms of the GNU General Public License as published 
//	by the Free Software Foundation; either version 2 of the 
//	License, or (at your option) any later version
// ==============================================================

#ifndef _SHARED_GRAPHICS_GRAPHICSINTERFACE_H_
#define _SHARED_GRAPHICS_GRAPHICSINTERFACE_H_

#include <map>
#include <string>

#include "vec.h"
#include "quaternion.h"
#include "camera.h"

#include "font.h"
#include "font_manager.h"

#include "pixmap.h"
#include "texture.h"
#include "texture_manager.h"

#include "text_renderer.h"

#include "model.h"
#include "model_manager.h"
#include "model_renderer.h"

#include "particle.h"
#include "particle_renderer.h"

#include "shader_manager.h"
#include "shader.h"

#include "graphics_factory.h"

#include "context.h"

using std::map;
using std::string;

namespace Shared{ namespace Graphics{

// =====================================================
//	class GraphicsInterface  
//
///	Interface for the graphic engine
// =====================================================

class GraphicsInterface{
private:
	GraphicsFactory *graphicsFactory;
	Context *currentContext;

private:
	friend class TextureManager;
	friend class FontManager;

private:
	GraphicsInterface();
	GraphicsInterface(GraphicsInterface &);
	void operator=(GraphicsInterface &);

public:
	static GraphicsInterface &getInstance();

	void setFactory(GraphicsFactory *graphicsFactory);
	void setCurrentContext(Context *context);	
	Context *getCurrentContext() const		{return currentContext;}

	ModelManager *newModelManager();
	TextureManager *newTextureManager();
	ParticleManager *newParticleManager();
	FontManager *newFontManager();
	ShaderManager *newShaderManager();

	TextRenderer2D *newTextRenderer2D();
	TextRenderer3D *newTextRenderer3D();
	ModelRenderer *newModelRenderer();
	ParticleRenderer *newParticleRenderer();
	
	Model *newModel();
	Texture1D *newTexture1D();
	Texture2D *newTexture2D();
	Texture3D *newTexture3D();
	TextureCube *newTextureCube();
	Font2D *newFont2D();
	Font3D *newFont3D();
	ShaderProgram *newShaderProgram();
	VertexShader *newVertexShader();
	FragmentShader *newFragmentShader();

	Context *newContext();
};

}}//end namespace

#endif
