// ==============================================================
//	This file is part of Glest Shared Library (www.glest.org)
//
//	Copyright (C) 2001-2005 Martio Figueroa
//
//	You can redistribute this code and/or modify it under 
//	the terms of the GNU General Public License as published 
//	by the Free Software Foundation; either version 2 of the 
//	License, or (at your option) any later version
// ==============================================================

#ifndef _SHARED_UTIL_FACTORY_
#define _SHARED_UTIL_FACTORY_

#include <map>
#include <string>
#include <stdexcept>

using std::map;
using std::pair;
using std::string;
using std::runtime_error;

namespace Shared{ namespace Util{

// =====================================================
//	class SingleFactoryBase
// =====================================================

template<typename T>
class SingleFactoryBase{
public:
	virtual T *newInstance()= 0;
};

// =====================================================
//	class SingleFactory
// =====================================================

template<typename T, typename S>
class SingleFactory: public SingleFactoryBase<S>{
public:
	virtual T *newInstance()	{return new T();}
};

// =====================================================
//	class MultiFactory
// =====================================================

template<typename T>
class MultiFactory{
private:
	typedef map<string, void*> Factories;
	typedef pair<string, void*> FactoryPair;

private:
	Factories factories;

public:
	~MultiFactory(){
		for(Factories::iterator it= factories.begin(); it!=factories.end(); ++it){
			delete static_cast<SingleFactoryBase<T>*>(it->second);
		}
	}

	template<typename S>
	void registerClass(const string &classId){
		factories.insert(FactoryPair(classId, new SingleFactory<S, T>()));
	}

	T *newInstance(const string &classId){
		Factories::iterator it= factories.find(classId);
		if(it == factories.end()){
			throw runtime_error("Unknown class identifier: " + classId);
		}
		return static_cast<T*>(static_cast<SingleFactoryBase<T> *>(it->second)->newInstance());
	}

	bool isClassId(const string &classId){
		return factories.find(classId)!=factories.end();
	}
};

}}//end namespace

#endif
