// ==============================================================
//	This file is part of Glest Shared Library (www.glest.org)
//
//	Copyright (C) 2001-2005 Martio Figueroa
//
//	You can redistribute this code and/or modify it under 
//	the terms of the GNU General Public License as published 
//	by the Free Software Foundation; either version 2 of the 
//	License, or (at your option) any later version
// ==============================================================

#include "graphics_interface.h"

#include <string>

#include "leak_dumper.h"

namespace Shared{ namespace Graphics{

// ===============================================
//	class GraphicsInterface
// ===============================================

GraphicsInterface::GraphicsInterface(){
	graphicsFactory= NULL;
}

GraphicsInterface &GraphicsInterface::getInstance(){
	static GraphicsInterface graphicsInterface;
	return graphicsInterface;
}

void GraphicsInterface::setFactory(GraphicsFactory *graphicsFactory){
	this->graphicsFactory= graphicsFactory;
}

void GraphicsInterface::setCurrentContext(Context *context){
	this->currentContext= context;
	currentContext->makeCurrent();
}

ModelManager *GraphicsInterface::newModelManager(){
	return new ModelManager();
}

TextureManager *GraphicsInterface::newTextureManager(){
	return new TextureManager();
}

ParticleManager *GraphicsInterface::newParticleManager(){
 	return new ParticleManager();
}

FontManager *GraphicsInterface::newFontManager(){
	return new FontManager();
}

ShaderManager *GraphicsInterface::newShaderManager(){
	return new ShaderManager();
}

TextRenderer2D *GraphicsInterface::newTextRenderer2D(){
	return graphicsFactory->newTextRenderer2D();
}

TextRenderer3D *GraphicsInterface::newTextRenderer3D(){
	return graphicsFactory->newTextRenderer3D();
}

ModelRenderer *GraphicsInterface::newModelRenderer(){
	return graphicsFactory->newModelRenderer();
}

ParticleRenderer *GraphicsInterface::newParticleRenderer(){
	return graphicsFactory->newParticleRenderer();
}

Model *GraphicsInterface::newModel(){
	return graphicsFactory->newModel();
}

Texture1D *GraphicsInterface::newTexture1D(){
	return graphicsFactory->newTexture1D();
}

Texture2D *GraphicsInterface::newTexture2D(){
	return graphicsFactory->newTexture2D();
}

Texture3D *GraphicsInterface::newTexture3D(){
	return graphicsFactory->newTexture3D();
}

TextureCube *GraphicsInterface::newTextureCube(){
	return graphicsFactory->newTextureCube();
}
	
Font2D *GraphicsInterface::newFont2D(){
	return graphicsFactory->newFont2D();
}

Font3D *GraphicsInterface::newFont3D(){
	return graphicsFactory->newFont3D();
}

Context *GraphicsInterface::newContext(){
	return graphicsFactory->newContext();
}

ShaderProgram *GraphicsInterface::newShaderProgram(){
	return graphicsFactory->newShaderProgram();
}

VertexShader *GraphicsInterface::newVertexShader(){
	return graphicsFactory->newVertexShader();
}

FragmentShader *GraphicsInterface::newFragmentShader(){
	return graphicsFactory->newFragmentShader();
}

}}//end namespace
